/////////////////////////////////////////////////////////////////
// History reads and stores a piecewise constant population history
// used for coalescent simulations. The history is made up of a series
// of epochs, each of which is defined by two parameters, theta (=2Nu)
// and tau (=2ut).

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.Reader;
import java.io.IOException;

public class History
{

    private static final int MAX_EPOCHS = 100;

    int numEpochs = 0;
    double[] thetaEpochs = new double[MAX_EPOCHS];
    double[] tauEpochs = new double[MAX_EPOCHS];

    // Set the number of epochs in the history
    public void setNumEpochs(int numEpochs)
    {
	if (numEpochs > MAX_EPOCHS) {
	    System.err.println("Too many epochs.");
	    System.exit(1);
	}

	this.numEpochs = numEpochs;
    }

    // Get the number of epochs in the history
    public int getNumEpochs() {
	return numEpochs;
    }


    // Set the parameters for a specified epoch
    public void setEpoch(int epochNum, double theta, double tau) {
	thetaEpochs[epochNum] = theta;
	tauEpochs[epochNum] = tau;
    }

    // Set the theta value for a specified epoch
    public void setTheta(int epochNum, double theta) {
	thetaEpochs[epochNum] = theta;
    }

    // Set the tau value for a specified epoch
    public void setTau(int epochNum, double tau) {
	tauEpochs[epochNum] = tau;
    }

    // Get the theta value for a specified epoch
    public double getTheta(int epochNum) {
	return thetaEpochs[epochNum];
    }

    // Get the tau value for a specified epoch
    public double getTau(int epochNum) {
	return tauEpochs[epochNum];
    }

    // Read the history from a specified file
    public void readHistory(String inFile) {

	int epoch = 0;

	// Set default input source to standard input
	Reader rdr = new InputStreamReader(System.in);

	// If a filename is given try to open the file
	if (!inFile.equals("")) {
	    try {
		rdr = new FileReader(inFile);
	    } catch (IOException e) {
		// If the file cannot be open, exit
		System.err.println("Please specify a valid history.");
		System.exit(1);
	    }
	}

        String line = "";

	try (BufferedReader br = new BufferedReader(rdr)) {

		while ((line = br.readLine()) != null) {

		    String[] thisLine = line.split(",");

		    // If line is a comment discard it and keep reading
		    if (thisLine[0].charAt(0) == '#')
			continue;

		    // Set the theta value for this epoch
		    thetaEpochs[epoch] = Double.parseDouble(thisLine[0]);

		    // Set the tau value for this epoch. If the tau is
		    // the string, "Inf." set it to Double.POSITIVE_INFINITY,
		    // otherwise set it to the specified value.
		    if (thisLine[1].equals("Inf.")) {
			tauEpochs[epoch] = Double.POSITIVE_INFINITY;
		    } else {
			tauEpochs[epoch] = Double.parseDouble(thisLine[1]);
		    }

		    numEpochs++; // Keep track of the number of epochs
		    epoch++; // A counter variable
		}

	    } catch (IOException e) {

	    e.printStackTrace();

	}
    }

}